import logging

from django.conf import settings
from django.db import transaction, connection
from django.db.models import PositiveIntegerField
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsQuerySet, CommonManagetWithLookUpFieldsManager
from isc_common.models.base_ref import Hierarcy
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.item import Item
from kaf_pas.production.models.operations import Operations

logger = logging.getLogger(__name__)


class Operations_itemQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operations_itemManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def refresh_num1():
        settings.LOCKS.acquire('Operations_itemManager.refresh_num1')
        sql = '''select item_id,
                           count(*) 
                    from production_operations_item
                    group by item_id
                    having count(*) = 1'''

        sql_max = f'''select num
                        from production_operations_item
                        where item_id = %s'''

        with transaction.atomic():
            cnt = 0
            with connection.cursor() as cursor:
                cursor.execute(sql)
                rows = cursor.fetchall()
                for row in rows:
                    item_id, count = row
                    cursor.execute(sql_max, [item_id])
                    rows1 = cursor.fetchall()

                    for row_num in rows1:
                        num, = row_num
                        if num == None:
                            cnt1 = Operations_item.objects.filter(item_id=item_id).count()
                            if cnt1 == 1:
                                operations_item = Operations_item.objects.get(item_id=item_id)
                                operations_item.num = 1
                                operations_item.save()
                                cnt += 1
            print(f'Corected: {cnt} operations.')
        settings.LOCKS.release('Operations_itemManager.refresh_num1')
        return cnt

    @staticmethod
    def refresh_num(apps, schema_editor):
        settings.LOCKS.acquire('Operations_itemManager.refresh_num')
        sql = '''select num,
                       item_id,
                       count(*) 
                from production_operations_item
                where num is not null
                group by num,
                         item_id
                having count(*) > 1'''

        sql_max = f'''select max(num)
                                from production_operations_item
                                where item_id = %s'''

        with transaction.atomic():
            with connection.cursor() as cursor:
                cursor.execute(sql)
                rows = cursor.fetchall()
                for row in rows:
                    num, item_id, count = row
                    cursor.execute(sql_max, [item_id])
                    row_max = cursor.fetchone()

                    num_max, = row_max

                    for operations_item in Operations_item.objects.filter(num=num, item_id=item_id):
                        num_max += 1
                        operations_item.num = num_max
                        operations_item.save()
        settings.LOCKS.release('Operations_itemManager.refresh_num')

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operation_id': record.operation.id,
            'operation__code': record.operation.code,
            'operation__name': record.operation.name,
            'operation__full_name': record.operation.full_name,
            'operation__description': record.operation.description,
            'ed_izm_id': record.ed_izm.id if record.ed_izm else None,
            'ed_izm__code': record.ed_izm.code if record.ed_izm else None,
            'ed_izm__name': record.ed_izm.name if record.ed_izm else None,
            'ed_izm__description': record.ed_izm.description if record.ed_izm else None,
            'qty': record.qty,
            'num': record.num,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operations_itemQuerySet(self.model, using=self._db)

    def _rec_def_resources(self, operationitem):
        from kaf_pas.production.models.operation_def_resources import Operation_def_resources
        from kaf_pas.production.models.operation_resources import Operation_resources
        for operation_def_resource in Operation_def_resources.objects.filter(operation=operationitem.operation):
            Operation_resources.objects.get_or_create(
                operationitem=operationitem,
                resource=operation_def_resource.resource,
                location=operation_def_resource.location,
            )

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'operation__full_name')

        oparations = _data.get('operation')
        item_id = _data.get('item_id')
        delAttr(_data, 'operation')
        res = []
        if isinstance(oparations, list):
            with transaction.atomic():
                if isinstance(item_id, int):
                    for oparation in oparations:
                        setAttr(_data, 'operation_id', oparation)
                        _res, created = super().get_or_create(**_data)
                        if created:
                            self._rec_def_resources(_res)
                            __res = model_to_dict(_res)
                            setAttr(__res, 'operation__full_name', _res.operation.full_name)
                            res.append(__res)
                elif isinstance(item_id, list):
                    delAttr(_data, 'item_id')
                    for oparation in oparations:
                        for item in item_id:
                            setAttr(_data, 'operation_id', oparation)
                            setAttr(_data, 'item_id', item)
                            _res, created = super().get_or_create(**_data)
                            if created:
                                self._rec_def_resources(_res)
                                __res = model_to_dict(_res)
                                setAttr(__res, 'operation__full_name', _res.operation.full_name)
                                res.append(__res)

        return res

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        if isinstance(_data, dict):
            for k, v in _data.items():
                data1 = dict()
                if isinstance(v, dict):
                    delAttr(v, 'operation__full_name')
                    for k1, v1 in v.items():
                        if k1.find('_') == -1:
                            setAttr(data1, k1, v1)
                    data1.setdefault('_operation', 'update')
                    res = super().filter(id=v.get('id')).update(**data1)
                    break
                else:
                    delAttr(_data, 'operation__full_name')
                    res = super().filter(id=data.get('id')).update(**_data)
                    break
        else:
            delAttr(_data, 'operation__full_name')
            res = super().filter(id=data.get('id')).update(**_data)

        return res


class Operations_item(Hierarcy):
    description = DescriptionField()
    item = ForeignKeyProtect(Item)
    operation = ForeignKeyProtect(Operations)
    num = PositiveIntegerField(default=None, null=True, blank=True)
    qty = PositiveIntegerField(default=None, null=True, blank=True)
    ed_izm = ForeignKeyProtect(Ed_izm, default=None, null=True, blank=True)

    objects = Operations_itemManager()

    def __str__(self):
        return f"ID:{self.id}, item: [{self.item}], id_izm: [{self.ed_izm}], qty: {self.qty}, num: {self.num}"

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('item', 'operation'),)



import logging

from django.db.models import EmailField
from django.utils.translation import gettext_lazy as _

from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel

logger = logging.getLogger(__name__)


class User_viewQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class User_viewManager(AuditManager):

    # @staticmethod
    # def getRecord(record):
    #     res = {}
    #     return res

    def get_queryset(self):
        return User_viewQuerySet(self.model, using=self._db)


class User_view(AuditModel):
    username = CodeField(verbose_name=_('логин'))
    first_name = NameField(verbose_name=_('имя'))
    last_name = NameField(verbose_name=_('фамилия'))
    email = EmailField(verbose_name=_('E-mail'), blank=True, null=True)
    middle_name = NameField(verbose_name=_('отчетво'))
    position_name = NameField(verbose_name=_('должность'))
    location_name = NameField(verbose_name=_('место работы'))

    objects = User_viewManager()

    def __str__(self):
        return f"ID: {self.id} username {self.username}"

    class Meta:
        verbose_name = 'Пользователи'
        db_table = 'user_view'
        managed = False



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from isc_common.logger.Logger import Logger
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_ext import DocumentManagerExt
from kaf_pas.kd.models.spws import Spws

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Command(BaseCommand):
    help = "Test Make SPW"
    documentManagerExt = DocumentManagerExt(logger=logger)

    def handle(self, *args, **options):
        spws = [spws for spws in Spws.objects.filter(props=~Documents.props.beenItemed) if spws.file_document.lower().find('мусор') == -1]

        with transaction.atomic():
            pbar = tqdm(total=len(spws))
            for document in spws:
                self.documentManagerExt.make_spw(document=document)
                pbar.update()

            pbar.close()



from django.utils.translation import ugettext_lazy as _

import logging

from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet, BaseRefHierarcy

logger = logging.getLogger(__name__)


class Operations_templateQuerySet(BaseRefQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operations_templateManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'parent': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operations_templateQuerySet(self.model, using=self._db)


class Operations_template(BaseRefHierarcy):
    objects = Operations_templateManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Шаблоны группы операций'



import logging

from django.conf import settings
from django.db import transaction
from django.forms import model_to_dict

from isc_common import delAttr
from isc_common.http.DSRequest import DSRequest
from isc_common.number import DelProps
from kaf_pas.planing.models.operations import Operations, OperationsManager, OperationsQuerySet
from kaf_pas.planing.models.operations_view import Operations_view, Operations_viewManager

logger = logging.getLogger(__name__)


class Write_off_itemsQuerySet(OperationsQuerySet):
    pass


class Write_off_itemsManager(OperationsManager):
    @staticmethod
    def getRecord(record):
        return Operations_viewManager.getRecord(record)

    def createFromRequest(self, request, removed=None):
        from kaf_pas.accounting.models.tmp_buffer import Tmp_buffer
        from kaf_pas.planing.models.operation_color import Operation_color
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_value import Operation_value

        request = DSRequest(request=request)
        data = request.get_data()
        user_id = request.user_id
        delAttr(data, 'items')
        _data = data.copy()
        _data.setdefault('opertype', settings.OPERS_TYPES_STACK.WRITE_OFF_TASK)
        _data.setdefault('creator_id', user_id)

        with transaction.atomic():
            parent = Operations.objects.create(**_data)
            Operation_refs.objects.create(child=parent)

            for item in Tmp_buffer.objects.filter(value_off__isnull=False).exclude(value_off=0):
                child = dict()
                child.setdefault('date', parent.date)
                child.setdefault('creator_id', user_id)
                child.setdefault('description', item.description)
                child.setdefault('opertype', settings.OPERS_TYPES_STACK.WRITE_DETAIL_OFF_TASK)
                child.setdefault('status', settings.OPERS_TYPES_STACK.POSTING_DETAIL_TASK_STATUSES.get('new'))
                child = Operations.objects.create(**child)

                Operation_refs.objects.create(parent=parent, child=child)

                if item.item:
                    Operation_item.objects.create(operation=child, item=item.item)

                if item.edizm:
                    Operation_value.objects.create(operation=child, value=item.value_off, edizm=item.edizm)

                if item.color:
                    Operation_color.objects.create(operation=child, color=item.color)

            Tmp_buffer.objects.filter(user_id=user_id).delete()

            parent = model_to_dict(parent)
            return DelProps(parent)

    def get_queryset(self):
        return Write_off_itemsQuerySet(self.model, using=self._db)


class Write_off_items(Operations_view):
    objects = Write_off_itemsManager()

    def __str__(self):
        return super().__str__()

    def __repr__(self):
        return self.__str__()

    class Meta:
        proxy = True
        verbose_name = 'Списания'



# Generated by Django 2.2 on 2019-04-18 10:20

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0028_auto_20190417_1833'),
    ]

    operations = [
        migrations.CreateModel(
            name='Item_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('STMP_1', isc_common.fields.name_field.NameField(verbose_name='Наименование изделия')),
                ('STMP_2', isc_common.fields.name_field.NameField(verbose_name='Обозначение документа')),
                ('relevant', isc_common.fields.code_field.CodeField(verbose_name='Актуальность документа')),
                ('parent_id', models.BigIntegerField()),
                ('props', models.BigIntegerField()),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Товарная позиция',
                'db_table': 'ckk_item_veiw',
                'managed': False,
            },
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-07 16:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0064_auto_20190918_0146'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations_item',
            name='num',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attrs_view

urlpatterns = [

    path('Document_attrs_view/Fetch/', document_attrs_view.Document_attrs_view_Fetch),
    path('Document_attrs_view/Add', document_attrs_view.Document_attrs_view_Add),
    path('Document_attrs_view/Update', document_attrs_view.Document_attrs_view_Update),
    path('Document_attrs_view/Remove', document_attrs_view.Document_attrs_view_Remove),
    path('Document_attrs_view/Lookup/', document_attrs_view.Document_attrs_view_Lookup),
    path('Document_attrs_view/Info/', document_attrs_view.Document_attrs_view_Info),
    path('Document_attrs_view/Copy', document_attrs_view.Document_attrs_view_Copy),

]



# Generated by Django 2.2.6 on 2019-10-11 16:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0081_auto_20190822_2250'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents_thumb',
            name='path',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='documents_thumb10',
            name='path',
            field=models.TextField(),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-24 10:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0036_auto_20190524_0932'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='demand',
            options={'verbose_name': 'Запуск'},
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-16 05:09

from django.db import migrations
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0107_auto_20190616_0450'),
    ]

    operations = [
        migrations.AddField(
            model_name='materials',
            name='code',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AddField(
            model_name='materials',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='materials',
            name='name',
            field=isc_common.fields.name_field.NameField(),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-28 11:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0007_auto_20190328_0705'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='parent',
            field=models.ManyToManyField(to='ckk.Item_line'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-13 20:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0098_uploades_lotsman_documents'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='lotsman_documents_attributes',
            name='attr_type',
        ),
        migrations.RemoveField(
            model_name='lotsman_documents_attributes',
            name='document',
        ),
        migrations.RemoveField(
            model_name='lotsman_documents_thumb',
            name='document',
        ),
        migrations.RemoveField(
            model_name='lotsman_documents_thumb10',
            name='document',
        ),
        migrations.AlterUniqueTogether(
            name='uploades_lotsman_documents',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='uploades_lotsman_documents',
            name='document',
        ),
        migrations.RemoveField(
            model_name='uploades_lotsman_documents',
            name='upload',
        ),
        migrations.DeleteModel(
            name='Lotsman_documents',
        ),
        migrations.DeleteModel(
            name='Lotsman_documents_attributes',
        ),
        migrations.DeleteModel(
            name='Lotsman_documents_thumb',
        ),
        migrations.DeleteModel(
            name='Lotsman_documents_thumb10',
        ),
        migrations.DeleteModel(
            name='Uploades_lotsman_documents',
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import material_askon

urlpatterns = [

    path('MaterialAskon/Fetch/', material_askon.Material_askon_Fetch),
    path('MaterialAskon/Add', material_askon.Material_askon_Add),
    path('MaterialAskon/Update', material_askon.Material_askon_Update),
    path('MaterialAskon/Remove', material_askon.Material_askon_Remove),
    path('MaterialAskon/Lookup/', material_askon.Material_askon_Lookup),
    path('MaterialAskon/Info/', material_askon.Material_askon_Info),
    path('MaterialAskon/Copy', material_askon.Material_askon_Copy),

]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.cdw_attrs import Cdw_attrs, Cdw_attrsManager


@JsonResponseWithException()
def Cdw_attrs_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Cdw_attrs.objects.
                select_related('attr_type').
                get_range_rows1(
                request=request,
                function=Cdw_attrsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Cdw_attrs_Add(request):
    return JsonResponse(DSResponseAdd(data=Cdw_attrs.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Cdw_attrs_Update(request):
    return JsonResponse(DSResponseUpdate(data=Cdw_attrs.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Cdw_attrs_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Cdw_attrs.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Cdw_attrs_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Cdw_attrs.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Cdw_attrs_Info(request):
    return JsonResponse(DSResponse(request=request, data=Cdw_attrs.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.2 on 2020-01-17 18:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0165_auto_20200117_1600'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations',
            name='planing_operation_type',
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 09:53

from django.db import migrations
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0046_auto_20190610_0953'),
        ('ckk', '0086_auto_20190609_2307'),
    ]

    operations = [
        migrations.RenameField(
            model_name='item_line',
            old_name='STMP_2',
            new_name='SPC_CLM_MARK',
        ),
        migrations.RenameField(
            model_name='item_line',
            old_name='STMP_1',
            new_name='SPC_CLM_NAME',
        ),
        migrations.AlterField(
            model_name='item_line',
            name='section',
            field=isc_common.fields.name_field.NameField(),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='subsection',
            field=isc_common.fields.name_field.NameField(),
        ),
        migrations.AlterUniqueTogether(
            name='item_line',
            unique_together={('SPC_CLM_NAME', 'SPC_CLM_MARK'), ('parent', 'child')},
        ),
    ]



from django.test import TestCase

from isc_common.auth.models.user import User
from kaf_pas.kd.models.pathes import Pathes


class Test_TestPathes(TestCase):
    def setUp(self):
        self.path = "path"
        self.user = User.objects.get(username='admin')

    def test_1(self):
        TestPathes_item = Pathes.objects.create(path=self.path)
        TestPathes_item = Pathes.objects.get(pk=TestPathes_item.id)
        self.assertEquals(TestPathes_item.path, self.path)
        self.assertEquals(TestPathes_item.parent, None)

        TestPathes_item_parent = TestPathes_item
        TestPathes_item = Pathes.objects.create(path=self.path,  parent=TestPathes_item_parent)
        TestPathes_item = Pathes.objects.get(pk=TestPathes_item.id)
        self.assertEquals(TestPathes_item.path, self.path)
        self.assertEquals(TestPathes_item.parent, TestPathes_item_parent)

    def test_2(self):
        self.path = "/path/path1/path2"
        TestPathes_item = Pathes.objects.create_ex(path=self.path, user=self.user, with_out_last=False)
        TestPathes_item = Pathes.objects.get(pk=TestPathes_item.id)
        self.assertEquals(TestPathes_item.absolute_path, self.path)

        self.path1 = 'E:/KAF/Кузова/К4310/К4310/К4310-232 Саратов, Радиан/К4310-232.50.00.000 - Кузов в сборе/К4310-232.52.00.000 - Выгородка/К4310-232.52.11.000 - Панель/К4310-232.52.11.000 - Панель.cdw'
        TestPathes_item = Pathes.objects.create_ex(path=self.path1, user=self.user, with_out_last=True)
        TestPathes_item = Pathes.objects.get(pk=TestPathes_item.id)
        self.assertEquals(TestPathes_item.absolute_path, '/KAF/Кузова/К4310/К4310/К4310-232 Саратов, Радиан/К4310-232.50.00.000 - Кузов в сборе/К4310-232.52.00.000 - Выгородка/К4310-232.52.11.000 - Панель')



# Generated by Django 2.2.8 on 2019-12-17 07:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0116_lotsman_documents_hierarcy_files'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='lotsman_documents_hierarcy_files',
            unique_together={('lotsman_document', 'path')},
        ),
    ]



from django.urls import path

from kaf_pas.production.views import operation_resources

urlpatterns = [

    path('Operation_resources/Fetch/', operation_resources.Operation_resources_Fetch),
    path('Operation_resources/Add', operation_resources.Operation_resources_Add),
    path('Operation_resources/Update', operation_resources.Operation_resources_Update),
    path('Operation_resources/Remove', operation_resources.Operation_resources_Remove),
    path('Operation_resources/Lookup/', operation_resources.Operation_resources_Lookup),
    path('Operation_resources/Info/', operation_resources.Operation_resources_Info),

]



# Generated by Django 2.2.6 on 2019-10-23 22:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0089_auto_20191023_2143'),
    ]

    operations = [
        migrations.RenameField(
            model_name='precent_items',
            old_name='precent',
            new_name='precent_dogovor',
        ),
        migrations.RenameField(
            model_name='precent_items',
            old_name='itemtype',
            new_name='precent_item_type',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-27 14:04

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0212_auto_20200225_1013'),
        ('production', '0215_auto_20200227_1111'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations_template_detail',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AddField(
            model_name='operations_template_detail',
            name='num',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='operations_template_detail',
            name='qty',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-08 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0134_auto_20200208_1419'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='lotsman_documents_hierarcy_refs',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(('parent', None), ('section', None), ('subsection', None)), fields=('child',), name='Lotsman_documents_hierarcy_refs_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(('section', None), ('subsection', None)), fields=('child', 'parent'), name='Lotsman_documents_hierarcy_refs_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(('parent', None), ('subsection', None)), fields=('child', 'section'), name='Lotsman_documents_hierarcy_refs_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(subsection=None), fields=('child', 'parent', 'section'), name='Lotsman_documents_hierarcy_refs_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(('parent', None), ('section', None)), fields=('child', 'subsection'), name='Lotsman_documents_hierarcy_refs_unique_constraint_4'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(section=None), fields=('child', 'parent', 'subsection'), name='Lotsman_documents_hierarcy_refs_unique_constraint_5'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(condition=models.Q(parent=None), fields=('child', 'section', 'subsection'), name='Lotsman_documents_hierarcy_refs_unique_constraint_6'),
        ),
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.UniqueConstraint(fields=('child', 'parent', 'section', 'subsection'), name='Lotsman_documents_hierarcy_refs_unique_constraint_7'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-22 07:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0076_auto_20191022_0722'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Operations_refs',
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.precent_dogovors import Precent_dogovors, Precent_dogovorsManager


@JsonResponseWithException()
def Precent_dogovors_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Precent_dogovors.objects.
                select_related('dogovor', 'precent').
                get_range_rows1(
                request=request,
                function=Precent_dogovorsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_dogovors_Add(request):
    return JsonResponse(DSResponseAdd(data=Precent_dogovors.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_dogovors_Update(request):
    return JsonResponse(DSResponseUpdate(data=Precent_dogovors.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_dogovors_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Precent_dogovors.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_dogovors_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Precent_dogovors.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_dogovors_Info(request):
    return JsonResponse(DSResponse(request=request, data=Precent_dogovors.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Precent_dogovors_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Precent_dogovors.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2 on 2019-04-13 10:03

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0004_auto_20190328_1018'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attrs',
            name='document',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Documents', verbose_name='Документ'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-29 06:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0047_precent_item_line_view'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='precent_item_line_view',
            table='precent_item_line_view',
        ),
    ]



import logging

from django.db.models import TextField, CharField

from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.input.models.user_positions import User_positions

logger = logging.getLogger(__name__)


class User_add_infoQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class User_add_infoManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'ip_address': record.ip_address,
            'comp_name': record.comp_name,
            'user_agent': record.user_agent,
            'color': record.color,
            'description': record.description,
            'phones': record.phones,
            'location_id': record.location.id,
            'location__code': record.location.code,
            'location__name': record.location.name,
            'location__full_name': record.location.full_name,
            'position_id': record.position.id if record.position else None,
            'position__code': record.position.code if record.position else None,
            'position__name': record.position.name if record.position else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return User_add_infoQuerySet(self.model, using=self._db)


class User_add_info(AuditModel):
    description = TextField(blank=True, null=True)
    comp_name = NameField()
    ip_address = CharField(max_length=20, null=True, blank=True)
    user_agent = TextField(null=True, blank=True)
    user = ForeignKeyProtect(User)
    location = ForeignKeyProtect(Locations)
    position = ForeignKeyProtect(User_positions, null=True, blank=True)
    color = CodeField()
    phones = DescriptionField()

    objects = User_add_infoManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Дополнительная пользовательская информация'



# Generated by Django 2.2.7 on 2019-11-28 12:29

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0161_auto_20191122_0920'),
        ('kd', '0085_auto_20191031_1745'),
        ('sales', '0120_tec_spc_item_refs'),
    ]

    operations = [
        migrations.AddField(
            model_name='tec_spc_item_refs',
            name='item_refs',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Item_refs'),
        ),
        migrations.CreateModel(
            name='Tec_spc_item_line',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('section', isc_common.fields.code_field.CodeStrictField()),
                ('section_num', models.PositiveIntegerField(blank=True, null=True)),
                ('subsection', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_COUNT', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_COUNT_tec', to='kd.Document_attributes', verbose_name='Количество')),
                ('SPC_CLM_FACTORY', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_FACTORY_tec', to='kd.Document_attributes', verbose_name='Предприятие - изготовитель')),
                ('SPC_CLM_FORMAT', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_FORMAT_tec', to='kd.Document_attributes', verbose_name='Формат')),
                ('SPC_CLM_KOD', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_KOD_tec', to='kd.Document_attributes', verbose_name='Код')),
                ('SPC_CLM_MARK', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MARK_tec', to='kd.Document_attributes', verbose_name='Обозначение')),
                ('SPC_CLM_MASSA', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MASSA_tec', to='kd.Document_attributes', verbose_name='Масса')),
                ('SPC_CLM_MATERIAL', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MATERIAL_tec', to='kd.Document_attributes', verbose_name='Материал')),
                ('SPC_CLM_NAME', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NAME_tec', to='kd.Document_attributes', verbose_name='Наименование')),
                ('SPC_CLM_NOTE', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NOTE_tec', to='kd.Document_attributes', verbose_name='Примечание')),
                ('SPC_CLM_POS', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_POS_tec', to='kd.Document_attributes', verbose_name='Позиция')),
                ('SPC_CLM_USER', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_USER_tec', to='kd.Document_attributes', verbose_name='Пользовательская')),
                ('SPC_CLM_ZONE', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_ZONE_tec', to='kd.Document_attributes', verbose_name='Зона')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_child_tec', to='ckk.Item', verbose_name='Товарная позиция')),
                ('item_line', isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Item_line')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_parent_tec', to='ckk.Item', verbose_name='Товарная позиция')),
            ],
            options={
                'verbose_name': 'Строка состава изделия',
                'ordering': ['section'],
                'unique_together': {('parent', 'child')},
            },
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-15 19:17

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0210_auto_20200212_0752'),
        ('production', '0180_ready_2_launch_detail_json'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ready_2_launch_detail',
            name='item',
        ),
        migrations.RemoveField(
            model_name='ready_2_launch_detail',
            name='json',
        ),
        migrations.AddField(
            model_name='ready_2_launch_detail',
            name='item_refs',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, to='ckk.Item_refs'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-29 15:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0009_auto_20191020_1914'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations_plan_refs',
            name='child',
        ),
        migrations.RemoveField(
            model_name='operations_plan_refs',
            name='parent',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='opertype',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='status',
        ),
        migrations.DeleteModel(
            name='Prod_task_status',
        ),
        migrations.RemoveField(
            model_name='prod_tasks',
            name='creator',
        ),
        migrations.RemoveField(
            model_name='prod_tasks',
            name='importances',
        ),
        migrations.RemoveField(
            model_name='prod_tasks',
            name='item',
        ),
        migrations.RemoveField(
            model_name='prod_tasks',
            name='status',
        ),
        migrations.RemoveField(
            model_name='operations_plan_view',
            name='operationsplan_ptr',
        ),
        migrations.DeleteModel(
            name='Operation_plan_types',
        ),
        migrations.DeleteModel(
            name='Operations_plan_refs',
        ),
        migrations.DeleteModel(
            name='OperationsPlan',
        ),
        migrations.DeleteModel(
            name='OperationsPlanStatus',
        ),
        migrations.DeleteModel(
            name='Prod_task_importances',
        ),
        migrations.DeleteModel(
            name='Prod_tasks',
        ),
        migrations.DeleteModel(
            name='Operations_plan_view',
        ),
    ]



import logging

from django.db import connection, transaction

from isc_common import Stack
from isc_common.bit import TurnBitOn
from isc_common.common.mat_views import create_tmp_mat_view, drop_mat_view
from isc_common.logger.Logger import Logger
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.production.models import progress_deleted

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Lotsman_documents_hierarcyManagerExt:
    def __init__(self, user, logger=logger, documentManagerExt=None):
        self.user = user
        self.documentManagerExt = documentManagerExt
        self.logger = logger

    def make_items(self, lotsman_documents_hierarcy_view_record):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.kd.models.document_attributes import Document_attributesManager
        from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
        from kaf_pas.kd.models.lotsman_documents_hierarcy_view import Lotsman_documents_hierarcy_view
        from kaf_pas.system.models.contants import Contants
        from kaf_pas.ckk.models.tmp_item_refs import Tmp_Item_refsManager

        items_pairs = Stack()

        mat_view_name = None
        try:
            imp_frpm_lotsman_label = 'Импорт из Лоцмана'

            top_item, created = Item.objects.get_or_create(
                STMP_1=Document_attributesManager.get_or_create_attribute(
                    attr_codes='STMP_1',
                    value_str=imp_frpm_lotsman_label,
                    logger=self.logger
                )[0],
                props=Item.props.relevant | Item.props.from_lotsman
            )

            item_refs, _ = Item_refs.objects.get_or_create(child=top_item)
            if  item_refs.parent != None:
                item_refs.delete()
                item_refs, _ = Item_refs.objects.get_or_create(child=top_item)

            item_top_level, _ = Contants.objects.update_or_create(
                code='top_level',
                defaults=dict(
                    name='Вершины товарных позиций')
            )
            const4, _ = Contants.objects.update_or_create(
                code='lotsman_top_level',
                defaults=dict(
                    parent=item_top_level,
                    name=imp_frpm_lotsman_label,
                    value=top_item.id)
            )

            if lotsman_documents_hierarcy_view_record.parent_id != None:
                while True:
                    try:
                        item = Item.objects.get(lotsman_document_id=lotsman_documents_hierarcy_view_record.id, )
                        top_item = item
                        break
                    except Item.DoesNotExist:
                        if lotsman_documents_hierarcy_view_record.parent_id != None:
                            lotsman_documents_hierarcy_view_record = Lotsman_documents_hierarcy_view.objects.get(id=lotsman_documents_hierarcy_view_record.parent_id)
                            if lotsman_documents_hierarcy_view_record.parent_id == None:
                                top_item.lotsman_document_id = lotsman_documents_hierarcy_view_record.id
                                if lotsman_documents_hierarcy_view_record._Version:
                                    top_item.version = lotsman_documents_hierarcy_view_record._Version.value_int,
                                if lotsman_documents_hierarcy_view_record._Type:
                                    top_item.lotsman_type_id = lotsman_documents_hierarcy_view_record._Type.value_int,
                                top_item.save()
                                break
                        else:
                            break
                    except Item.MultipleObjectsReturned:
                        item = Item.objects.filter(lotsman_document_id=lotsman_documents_hierarcy_view_record.id, )[0]
                        top_item = item
                        break

            parent_str = f'''parent_id ={lotsman_documents_hierarcy_view_record.id}'''
            props_str = f'props in (1, 3)'

            sql_str = f'''WITH RECURSIVE r AS (
                               SELECT *, 1 AS level
                               FROM kd_lotsman_documents_hierarcy_mview
                               WHERE {parent_str}
                                     and  {props_str}  

                               union all

                               SELECT kd_lotsman_documents_hierarcy_mview.*, r.level + 1 AS level
                               FROM kd_lotsman_documents_hierarcy_mview
                                   JOIN r
                               ON kd_lotsman_documents_hierarcy_mview.parent_id = r.id)

                           select * from r where {props_str} order by level'''

            mat_view_name = create_tmp_mat_view(sql_str=sql_str, indexes=['attr_name', 'parent_id'])

            with connection.cursor() as cursor:
                cursor.execute(f'select count(*) from {mat_view_name}')
                cnt, = cursor.fetchone()

            # logger.logging(f'''SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
            # logger.logging(f'qty: {cnt}', 'debug')

            if cnt > 0:
                with managed_progress(
                        id=lotsman_documents_hierarcy_view_record.id,
                        qty=cnt,
                        user=self.user,
                        message=f'''Создание товарных позиций ({lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str} {cnt} шт.)''',
                        title='Выполнено',
                        props=TurnBitOn(0, 0)
                ) as progress:
                    try:
                        with transaction.atomic():

                            Tmp_Item_refsManager.create()
                            for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} order by level'''):
                                if lotsman_documents_hierarcy.attr_name not in ['Материал']:

                                    item, created1 = self.get_item(
                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                                        items_pairs=items_pairs,
                                    )

                                    if lotsman_documents_hierarcy.parent_id:
                                        try:
                                            _lotsman_documents_hierarcy = Lotsman_documents_hierarcy_view.objects.get(id=lotsman_documents_hierarcy.parent_id)
                                        except Lotsman_documents_hierarcy_view.DoesNotExist as ex:
                                            raise ex
                                        except Lotsman_documents_hierarcy_view.MultipleObjectsReturned:
                                            _lotsman_documents_hierarcy = Lotsman_documents_hierarcy_view.objects.filter(id=lotsman_documents_hierarcy.parent_id).distinct('id')[0]

                                        parent, created2 = self.get_item(
                                            lotsman_documents_hierarcy=_lotsman_documents_hierarcy,
                                            items_pairs=items_pairs,
                                        )

                                        item_refs, created2 = Item_refs.objects.get_or_create(parent=parent, child=item)

                                    # FOR DEBUG
                                    # if created == True or created1 == True or created2 == True:
                                    #     pass
                                    # END FOR DEBUG

                                Lotsman_documents_hierarcy.objects.update_or_create(
                                    id=lotsman_documents_hierarcy.id,
                                    defaults=dict(
                                        props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
                                    ))

                                res = progress.step()
                                if res != 0:
                                    raise ProgressDroped(progress_deleted)

                            with connection.cursor() as cursor:
                                cursor.execute(f'''select count(*) from {mat_view_name} where attr_name=%s''', ['''Сборочная единица'''])
                                assmbl_progress_count, = cursor.fetchone()

                            with managed_progress(
                                    id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
                                    qty=assmbl_progress_count,
                                    user=self.user,
                                    message=f'''Создание сборочных единиц. ({assmbl_progress_count}  шт.)''',
                                    title='Выполнено',
                                    # props=TurnBitOn(0, 0)
                            ) as assmbl_progress:

                                # logger.logging(f'''Сборочные единицы SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
                                # logger.logging(f'qty: {assmbl_progress_count}', 'debug')

                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where attr_name=%s order by level''', ['''Сборочная единица''']):
                                    assmbl_progress.setContentsLabel(f'''<h3>Создание сборочной единицы. ({lotsman_documents_hierarcy.SPC_CLM_NAME.value_str if lotsman_documents_hierarcy.SPC_CLM_NAME else ''} : 
                                                                                                             {lotsman_documents_hierarcy.SPC_CLM_MARK.value_str if lotsman_documents_hierarcy.SPC_CLM_MARK else ''})</h3>''')
                                    item, _ = self.get_item(
                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                                        items_pairs=items_pairs,
                                    )

                                    self.make_lines(
                                        parent=item,
                                        items_pairs=items_pairs,
                                        mat_view_name=mat_view_name,
                                        user_id=self.user,
                                        record=lotsman_documents_hierarcy_view_record,
                                    )

                                    assmbl_progress.step()
                                    res = progress.step()
                                    if res != 0:
                                        raise ProgressDroped(progress_deleted)

                            with connection.cursor() as cursor:
                                cursor.execute(f'''select count(*) from {mat_view_name} where attr_name=%s''', ['''Чертеж'''])
                                assmbl_progress_count, = cursor.fetchone()

                            with managed_progress(
                                    id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
                                    qty=assmbl_progress_count,
                                    user=self.user,
                                    message=f'''Привязка чертежей. ({assmbl_progress_count}  шт.)''',
                                    title='Выполнено',
                                    # props=TurnBitOn(0, 0)
                            ) as assmbl_progress:

                                # logger.logging(f'''Чертежи SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
                                # logger.logging(f'qty: {assmbl_progress_count}', 'debug')

                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where attr_name=%s order by level''', ['''Чертеж''']):

                                    assmbl_progress.setContentsLabel(f'''<h3>Привязка чертежей. ({lotsman_documents_hierarcy.SPC_CLM_NAME.value_str if lotsman_documents_hierarcy.SPC_CLM_NAME else ''} : 
                                                                                                    {lotsman_documents_hierarcy.SPC_CLM_MARK.value_str if lotsman_documents_hierarcy.SPC_CLM_MARK else ''})</h3>''')
                                    item, _ = self.get_item(
                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                                        items_pairs=items_pairs,
                                    )

                                    for item_ref in Item_refs.objects.filter(child=item):
                                        if item_ref.parent.lotsman_document.attr_type.code != 'Сборочная единица':
                                            self.documentManagerExt.link_image_to_lotsman_item(
                                                lotsman_document=lotsman_documents_hierarcy,
                                                item=item_ref.parent,
                                            )
                                            item_ref.delete()

                                    assmbl_progress.step()

                                    res = progress.step()
                                    if res != 0:
                                        raise ProgressDroped(progress_deleted)

                            with managed_progress(
                                    id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
                                    qty=assmbl_progress_count,
                                    user=self.user,
                                    message=f'''Перемещение чертежей. ({assmbl_progress_count}  шт.)''',
                                    title='Выполнено',
                                    # props=TurnBitOn(0, 0)
                            ) as assmbl_progress:

                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where attr_name=%s order by level''', ['''Чертеж''']):
                                    item, _ = self.get_item(
                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                                        items_pairs=items_pairs,
                                    )

                                    if Item_refs.objects.filter(child=item).count() == 0 and Item_refs.objects.filter(parent=item).count() == 0:
                                        item.delete()

                                    assmbl_progress.step()

                                    res = progress.step()
                                    if res != 0:
                                        raise ProgressDroped(progress_deleted)

                    except ProgressDroped as ex:
                        Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                        raise ex

            drop_mat_view(mat_view_name)
        except Exception as ex:
            drop_mat_view(mat_view_name)
            raise ex

    def make_lines(self, parent, items_pairs, mat_view_name, user_id, record):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.kd.models.lotsman_documents_hierarcy_view import Lotsman_documents_hierarcy_view
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_line import Item_lineManager

        if not isinstance(parent, Item):
            raise Exception(f'item mut be Item instnce.')

        with connection.cursor() as cursor:
            cursor.execute(f'select count(*) from {mat_view_name} where parent_id=%s', [parent.lotsman_document.id])
            count, = cursor.fetchone()

        # logger.logging(f'parent.lotsman_document: {parent.lotsman_document}', 'debug')
        # logger.logging(f'qty: {count}', 'debug')

        with managed_progress(
                id=f'''lines_{parent.id}''',
                qty=count,
                user=user_id,
                message=f'''Создание строк детализации. ({count} шт.)''',
                title='Выполнено'
        ) as lines_progress:
            for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'select * from {mat_view_name} where parent_id=%s order by level', [parent.lotsman_document.id]):
                item, _ = self.get_item(
                    lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                    items_pairs=items_pairs,
                )

                if lotsman_documents_hierarcy.attr_name != 'Материал':

                    if lotsman_documents_hierarcy.section == None:
                        if lotsman_documents_hierarcy.attr_name in ['Чертеж', 'Спецификация']:
                            lotsman_documents_hierarcy.section = 'Документация'
                        elif lotsman_documents_hierarcy.attr_name in ['Сборочная единица']:
                            lotsman_documents_hierarcy.section = 'Сборочные единицы'
                        elif lotsman_documents_hierarcy.attr_name in ['Деталь']:
                            lotsman_documents_hierarcy.section = 'Детали'

                    if lotsman_documents_hierarcy.section == None:
                        lotsman_documents_hierarcy.section = lotsman_documents_hierarcy.attr_name

                    Документ_на_материал = None
                    Наименование_материала = None
                    Документ_на_сортамент = None
                    Наименование_сортамента = None

                    for lotsman_documents_hierarcy_view in Lotsman_documents_hierarcy_view.objects.filter(parent_id=lotsman_documents_hierarcy.id):
                        if lotsman_documents_hierarcy_view.attr_name in ['Материал']:
                            Документ_на_материал = lotsman_documents_hierarcy_view.Документ_на_материал
                            Наименование_материала = lotsman_documents_hierarcy_view.Наименование_материала
                            Документ_на_сортамент = lotsman_documents_hierarcy_view.Документ_на_сортамент
                            Наименование_сортамента = lotsman_documents_hierarcy_view.Наименование_сортамента
                            break

                    item_line, created = Item_line.objects.update_or_create(
                        parent=parent,
                        child=item,
                        defaults=dict(
                            SPC_CLM_FORMAT=lotsman_documents_hierarcy.SPC_CLM_FORMAT,
                            SPC_CLM_ZONE=lotsman_documents_hierarcy.SPC_CLM_ZONE,
                            SPC_CLM_POS=lotsman_documents_hierarcy.SPC_CLM_POS,
                            SPC_CLM_MARK=lotsman_documents_hierarcy.SPC_CLM_MARK,
                            SPC_CLM_NAME=lotsman_documents_hierarcy.SPC_CLM_NAME,
                            SPC_CLM_COUNT=lotsman_documents_hierarcy.SPC_CLM_COUNT,
                            SPC_CLM_NOTE=lotsman_documents_hierarcy.SPC_CLM_NOTE,
                            SPC_CLM_MASSA=lotsman_documents_hierarcy.SPC_CLM_MASSA,
                            SPC_CLM_MATERIAL=lotsman_documents_hierarcy.SPC_CLM_MATERIAL if lotsman_documents_hierarcy.SPC_CLM_MATERIAL else Наименование_материала,
                            SPC_CLM_USER=lotsman_documents_hierarcy.SPC_CLM_USER,
                            SPC_CLM_KOD=lotsman_documents_hierarcy.SPC_CLM_KOD,
                            SPC_CLM_FACTORY=lotsman_documents_hierarcy.SPC_CLM_FACTORY,
                            Документ_на_материал=Документ_на_материал,
                            Наименование_материала=Наименование_материала,
                            Документ_на_сортамент=Документ_на_сортамент,
                            Наименование_сортамента=Наименование_сортамента,
                            section=lotsman_documents_hierarcy.section,
                            section_num=Item_lineManager.section_num(lotsman_documents_hierarcy.section),
                            subsection=lotsman_documents_hierarcy.subsection,
                        )
                    )

                lines_progress.step()

    def get_item(self, lotsman_documents_hierarcy, items_pairs):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs

        items = [item[1] for item in items_pairs.stack if item[0] == lotsman_documents_hierarcy.id]
        if len(items) == 0:
            STMP_1 = lotsman_documents_hierarcy.SPC_CLM_NAME
            STMP_2 = lotsman_documents_hierarcy.SPC_CLM_MARK

            item, created = Item.objects.get_or_create(
                lotsman_document_id=lotsman_documents_hierarcy.id,
                version=lotsman_documents_hierarcy._Version.value_int if lotsman_documents_hierarcy._Version else None,
                lotsman_type_id=lotsman_documents_hierarcy._Type.value_int if lotsman_documents_hierarcy._Type else None,
                defaults=dict(
                    STMP_1=STMP_1,
                    STMP_2=STMP_2,
                    props=Item.props.relevant | Item.props.from_lotsman,
                )
            )

            if lotsman_documents_hierarcy.attr_name in ['Деталь']:
                for lotsman_documents_hierarcy_chert in Lotsman_documents_hierarcy_refs.objects.filter(
                        parent_id=lotsman_documents_hierarcy.id,
                        child__attr_type__code='Чертеж'):
                    self.documentManagerExt.link_image_to_lotsman_item(
                        lotsman_document=lotsman_documents_hierarcy_chert.child,
                        item=item,
                    )
            else:
                self.documentManagerExt.link_image_to_lotsman_item(
                    lotsman_document=lotsman_documents_hierarcy,
                    item=item,
                )

            items_pairs.push((lotsman_documents_hierarcy.id, item))

            # if logger and created:
            #     logger.logging(f'\nAdded parent: {item}', 'debug')

            return item, created
        elif len(items) == 1:
            return items[0], False
        else:
            raise Exception(f'Неоднозначный выбор.')

    def make_mview(self):
        from kaf_pas.system.models.contants import Contants
        from django.db import connection

        index_sql = []
        fields_sql = []
        sql_array = []

        parent_system_const, _ = Contants.objects.update_or_create(
            code='lotsman_attibutes',
            defaults=dict(name='Атрибуты товарных позиций импортированных из Лоцмана')
        )

        attr_map = {
            'Зона': 'SPC_CLM_ZONE',
            'Код': 'SPC_CLM_KOD',
            'Масса': 'SPC_CLM_MASSA',
            'Материал': 'SPC_CLM_MATERIAL',
            'Наименование': 'SPC_CLM_NAME',
            'Обозначение': 'SPC_CLM_MARK',
            'Позиция': 'SPC_CLM_POS',
            'Пользовательская': 'SPC_CLM_USER',
            'Предприятие - изготовитель': 'SPC_CLM_FACTORY',
            'Примечание': 'SPC_CLM_NOTE',
            'Формат': 'SPC_CLM_FORMAT',
        }

        for name, code in attr_map.items():
            Contants.objects.update_or_create(
                code=code,
                defaults=dict(
                    name=name,
                    parent=parent_system_const
                )
            )

        m_view_name = 'kd_lotsman_documents_hierarcy_mview'
        m_view_recurs_name = 'kd_lotsman_documents_hierarcy_recurs_view'

        sql_array.append(f'DROP VIEW IF EXISTS {m_view_recurs_name} CASCADE')
        sql_array.append(f'DROP MATERIALIZED VIEW IF EXISTS {m_view_name} CASCADE')
        sql_array.append(f'''CREATE MATERIALIZED VIEW {m_view_name} AS SELECT lts.id,
                                                                                      lts.deleted_at,
                                                                                      lts.editing,
                                                                                      lts.deliting,
                                                                                      lts.lastmodified,                                                                                
                                                                                      ltsr.parent_id,
                                                                                      lts.props,
                                                                                      lts.attr_type_id,
                                                                                      lts.document_id,
                                                                                      CASE
                                                                                           WHEN (select count(1) as count
                                                                                                 from kd_lotsman_documents_hierarcy_refs hr                                                            	
                                                                                                 where hr.parent_id = lts.id) > 0 THEN true
                                                                                           ELSE false
                                                                                      END AS "isFolder",                                                                                 
                                                                                      ltsr.section, 
                                                                                      ltsr.subsection,
                                                                                      att.code attr_code,
                                                                                      att.name attr_name
                                                                                      $COMMA
                                                                                      $FIELDS     
                                                                               FROM kd_lotsman_documents_hierarcy lts
                                                                                       join kd_lotsman_documents_hierarcy_refs ltsr on ltsr.child_id = lts.id
                                                                                       join ckk_attr_type att on att.id = lts.attr_type_id WITH DATA;
                                                                                       --join ckk_attr_type att on att.id = lts.attr_type_id ;
                                                                               $INDEXES''')

        for field in ['id', 'deleted_at', 'editing', 'deliting', 'lastmodified', 'parent_id', 'props', 'attr_type_id', 'document_id', 'isFolder', 'section', 'subsection', 'attr_code', 'attr_name']:
            index_sql.append(f'''CREATE INDEX "ldh_attr_{field}_idx" ON {m_view_name} USING btree ("{field}")''')

        for field in Contants.objects.filter(parent__code='lotsman_attibutes'):
            fields_sql.append(f'''( SELECT kat.id
                                                  FROM kd_document_attributes kat
                                                    JOIN kd_lotsman_document_attr_cross dc ON kat.id = dc.attribute_id
                                                    JOIN ckk_attr_type att ON att.id = kat.attr_type_id
                                                 WHERE dc.document_id = ltsr.child_id and (dc.parent_document_id = ltsr.parent_id or ltsr.parent_id is null) AND att.code::text = '{field.code}'::text limit 1) AS "{field.code}_id"''')
            index_sql.append(f'''CREATE INDEX "ldh_attr_{field.code}_idx" ON {m_view_name} USING btree ("{field.code}_id")''')

        if len(fields_sql) > 0:
            sql_str = ';\n'.join(sql_array).replace('$FIELDS', ',\n'.join(fields_sql)).replace('$INDEXES', ';\n'.join(index_sql))
            sql_str = sql_str.replace('$COMMA', ',')
        else:
            sql_str = ';\n'.join(sql_array).replace('$FIELDS', '')
            sql_str = sql_str.replace('$COMMA', '')

        with connection.cursor() as cursor:
            # self.logger.logging(f'\n{sql_str}')
            cursor.execute(sql_str)
            # self.logger.logging(f'{m_view_name} recreated')

            sql_array = []
            sql_array.append(f'''CREATE VIEW {m_view_recurs_name} AS select * from (WITH RECURSIVE r AS (
                                       SELECT *, 1 AS level
                                       FROM {m_view_name}
                                       WHERE parent_id IS NULL

                                       union all

                                       SELECT {m_view_name}.*, r.level + 1 AS level
                                       FROM {m_view_name}
                                                JOIN r
                                                     ON {m_view_name}.parent_id = r.id)                                
                                   select * from r order by level) as a''')
            sql_str = ';\n'.join(sql_array)
            # self.logger.logging(f'\n{sql_str}')
            cursor.execute(sql_str)
            # self.logger.logging(f'{m_view_recurs_name} recreated')



# Generated by Django 3.0.3 on 2020-02-08 14:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0170_auto_20200131_1114'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='launch_item_refs',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='launch_item_refs',
            constraint=models.UniqueConstraint(condition=models.Q(parent=None), fields=('child', 'launch'), name='Launch_item_refs_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='launch_item_refs',
            constraint=models.UniqueConstraint(fields=('child', 'launch', 'parent'), name='Launch_item_refs_unique_constraint_1'),
        ),
    ]



from django.urls import path

from kaf_pas.sales.views import precent

urlpatterns = [

    path('Precent/Fetch/', precent.Precent_Fetch),
    path('Precent/Add', precent.Precent_Add),
    path('Precent/Update', precent.Precent_Update),
    path('Precent/Remove', precent.Precent_Remove),
    path('Precent/Lookup/', precent.Precent_Lookup),
    path('Precent/Info/', precent.Precent_Info),
    path('Precent/Upload', precent.Precent_UploadFile),
    path('Precent/Download/', precent.Precent_DownloadFile),
]



# Generated by Django 2.2.1 on 2019-05-30 11:59

from django.db import migrations
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0041_auto_20190530_1155'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='code',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AddField(
            model_name='operations',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='operations',
            name='name',
            field=isc_common.fields.name_field.NameField(),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-29 13:08

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0069_materials_material_1c'),
        ('production', '0035_operation_resources'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operation_material',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('material', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Materials')),
                ('operation', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operations')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-20 07:29

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0066_delete_documents_history'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document_attributes',
            name='props',
        ),
        migrations.AddField(
            model_name='document_attr_cross',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('copied', 'Копированный')), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='documents',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('beenItemed', 'Был внесен в состав изделий'), ('modified', 'Был модифицирован')), db_index=True, default=1),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_level

urlpatterns = [

    path('Operation_level/Fetch/', operation_level.Operation_level_Fetch),
    path('Operation_level/Add', operation_level.Operation_level_Add),
    path('Operation_level/Update', operation_level.Operation_level_Update),
    path('Operation_level/Remove', operation_level.Operation_level_Remove),
    path('Operation_level/Lookup/', operation_level.Operation_level_Lookup),
    path('Operation_level/Info/', operation_level.Operation_level_Info),
    path('Operation_level/Copy', operation_level.Operation_level_Copy),

]



# Generated by Django 2.2.3 on 2019-07-14 09:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0058_auto_20190714_0852'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='uploades_documents',
            unique_together={('upload', 'document')},
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-03 05:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('capacity', '0003_remove_resources_id_old'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Resources',
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-09 02:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0049_demand_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='long_name',
            field=models.TextField(blank=True, null=True, verbose_name='Полное наименование'),
        ),
        migrations.RunSQL(
            f"""update sales_customer set long_name = full_name"""
        ),
    ]



from django.urls import path

from kaf_pas.sales.views import precent_items

urlpatterns = [

    path('Precent_items/Fetch/', precent_items.Precent_items_Fetch),
    path('Precent_items/Add', precent_items.Precent_items_Add),
    path('Precent_items/Update', precent_items.Precent_items_Update),
    path('Precent_items/Remove', precent_items.Precent_items_Remove),
    path('Precent_items/Lookup/', precent_items.Precent_items_Lookup),
    path('Precent_items/Info/', precent_items.Precent_items_Info),

]



# Generated by Django 3.0.2 on 2020-01-19 07:59

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0166_remove_operations_planing_operation_type'),
        ('planing', '0062_operations_description'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_launches',
            name='launch',
        ),
        migrations.AddField(
            model_name='operation_launches',
            name='launch_operations_item',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='planing_resource_launch', to='production.Launch_operations_item'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 20:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0142_resource_users_props'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations_item',
            name='q_paralel',
            field=models.PositiveIntegerField(default=1),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-28 14:06

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0079_auto_20191028_1405'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations_item',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 20:43

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0098_launches_view'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='operation_resources',
            field=isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='production.Operation_resources'),
        ),
        migrations.AlterField(
            model_name='launch_operations_material',
            name='operation_material',
            field=isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='production.Operation_material'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-22 10:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0105_operations_view_std_material'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operations_view',
            options={'managed': False, 'verbose_name': 'Опреации системные'},
        ),
    ]



import configparser
import logging
import os
from datetime import datetime
from os.path import getmtime
from tkinter import *
from tkinter import filedialog
from xml.dom.expatbuilder import TEXT_NODE
from xml.dom.minidom import parseString

from django.core.management import BaseCommand
from django.db import transaction
from isc_common.common.mat_views import refresh_mat_view
from tqdm import tqdm

from isc_common import setAttr
from kaf_pas.ckk.models.files_askon import Files_askon
from kaf_pas.ckk.models.material_askon import Material_askon

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Загрузка материалов и АСКОНа"
    config = configparser.ConfigParser()
    config['DEFAULT'] = {}

    config.sections()
    config.read('config.ini')

    root = Tk()

    def is_text_node(self, node):
        if len(node.childNodes) > 1:
            return None, None
        elif len(node.childNodes) == 1:
            child = node.childNodes[0]
            if child.nodeType == TEXT_NODE:
                tag_name = node.tagName
                value = child.data
                return tag_name, value
            else:
                return None, None
        else:
            raise Exception(f"Unknown case.")

    def rec_node(self, node, parent=None):
        text_nodes = dict()
        elements = []
        for child in node.childNodes:
            tag_name, value = self.is_text_node(child)
            if tag_name and value:
                setAttr(text_nodes, tag_name, value)
            elif not tag_name and not value:
                elements.append(child)

        if bool(text_nodes):
            if parent:
                setAttr(text_nodes, 'parent_id', parent.id)
            parent, created = Material_askon.objects.update_or_create(**text_nodes)
            if created:
                logger.debug(f'created: {parent}')
            else:
                logger.debug(f'updated: {parent}')
        else:
            parent = None

        for element in elements:
            self.rec_node(element, parent)

    def handle(self, *args, **options):

        logger.info(self.help)

        try:
            initialdir = self.config['DEFAULT']['askondir']
            if not os.path.exists(initialdir):
                initialdir = None
        except KeyError:
            initialdir = None

        self.root.filename = filedialog.askopenfilename(title="Выберите файл документов", initialdir=initialdir)
        if len(self.root.filename) == 0:
            return

        dir, _ = os.path.split(self.root.filename)
        self.config['DEFAULT']['askondir'] = dir

        with open('config.ini', 'w') as configfile:
            self.config.write(configfile)

        logger.debug(f'filename : {self.root.filename}')
        self.root.withdraw()

        date_modification = datetime.fromtimestamp(getmtime(self.root.filename))
        with transaction.atomic():

            file, created = Files_askon.objects.get_or_create(real_name=self.root.filename, file_modification_time=date_modification, props=Files_askon.props.imp)

            if created:
                file = open(self.root.filename, 'r')
                data = file.read()
                file.close()
                dom = parseString(data)

                rows = dom.getElementsByTagName('rows')
                qty = len(rows[0].childNodes)
                self.pbar = tqdm(total=qty)
                for child in rows[0].childNodes:
                    self.rec_node(child)

                    if self.pbar:
                        self.pbar.update(1)

                if self.pbar:
                    self.pbar.close()

                refresh_mat_view('ckk_material_askon_mview')

            else:
                logger.warning(f'Файл: {self.root.filename}, уже импортирован !')

        logger.info("Загрузка выполнена.")



# Generated by Django 2.2.8 on 2019-12-25 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0163_launches_props'),
    ]

    operations = [
        migrations.RenameField(
            model_name='operation_resources',
            old_name='q_paralel',
            new_name='batch_size',
        ),
        migrations.RemoveField(
            model_name='launch_operation_resources',
            name='q_paralel',
        ),
        migrations.AddField(
            model_name='launch_operation_resources',
            name='batch_size',
            field=models.PositiveIntegerField(default=1),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-21 22:23

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0133_auto_20191121_2116'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operations_material',
            name='edizm',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
    ]



import logging

from django.core.management import BaseCommand

from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_view import Item_view

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def handle(self, *args, **options):
        logger.info(self.help)

        query = Item.objects.filter(props=Item.props.from_cdw)
        query = Item_view.objects.filter(from_cdw=True)
        for item in query:
            logger.debug(item)



from django.urls import path

from kaf_pas.ckk.views import item, material_type

urlpatterns = [

    path('Item/Fetch/', item.Item_Fetch),
    path('Item/Add', item.Item_Add),
    path('Item/Update', item.Item_Update),
    path('Item/Remove', item.Item_Remove),
    path('Item/Lookup/', item.Item_Lookup),
    path('Item/Info/', item.Item_Info),
    path('Item/CheckRecursives/', item.Item_CheckRecursives),
    path('Item/GetQtyChilds/', item.GetQtyChilds),
]



import logging

from django.db.models import TextField

from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.kd.models.uploades import Uploades

logger = logging.getLogger(__name__)


class Uploades_logQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Uploades_logManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'upload_id': record.upload.id,
            'lastmodified': record.lastmodified,
            'log': record.log,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Uploades_logQuerySet(self.model, using=self._db)


class Uploades_log(AuditModel):
    log = TextField()
    upload = ForeignKeyCascade(Uploades)

    objects = Uploades_logManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Строки лога закачки'



# Generated by Django 2.2.1 on 2019-05-23 16:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0023_precent_operation_plan_templates'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent',
            name='operation_plan_templates',
        ),
    ]



from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.accounting.models.tmp_buffer import Tmp_buffer, Tmp_bufferManager


@JsonResponseWithException()
def Tmp_buffer_Fetch(request):
    _request = DSRequest(request=request)

    return JsonResponse(
        DSResponse(
            request=request,
            data=Tmp_buffer.objects.
                filter(user_id=_request.user_id).
                filter().
                get_range_rows1(
                request=request,
                function=Tmp_bufferManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_Add(request):
    return JsonResponse(DSResponseAdd(data=Tmp_buffer.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_Update(request):
    return JsonResponse(DSResponseUpdate(data=Tmp_buffer.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_Info(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2 on 2019-04-25 17:05

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0039_auto_20190425_1146'),
        ('production', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operation_plan_template_locations',
            options={'verbose_name': 'место положение для операций планирования'},
        ),
        migrations.AddField(
            model_name='operation_plan_template_locations',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AddField(
            model_name='operation_plan_template_locations',
            name='operation_plan_templates',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_templates'),
        ),
    ]



import logging

from django.db import transaction
from django.db.models import DecimalField, UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import setAttr, delAttr
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import Hierarcy
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.material_askon import Material_askon
from kaf_pas.ckk.models.materials import Materials
from kaf_pas.production.models.operations_template_detail import Operations_template_detail

logger = logging.getLogger(__name__)


class Operations_template_materialQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        if kwargs.get('material'):
            setAttr(kwargs, 'material_id', kwargs.get('material').id)
            delAttr(kwargs, 'material')

        if kwargs.get('material_askon'):
            setAttr(kwargs, 'material_askon_id', kwargs.get('material_askon').id)
            delAttr(kwargs, 'material_askon')

        if not kwargs.get('material_id') and not kwargs.get('material_askon_id'):
            raise Exception('Необходим хотябы один выбранный параметр.')
        setAttr(kwargs, '_operation', 'create')
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operations_template_materialManager(CommonManagetWithLookUpFieldsManager):

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        template_id = _data.get('template_id')

        res = []
        if isinstance(template_id, list):
            delAttr(_data, 'template_id')
            with transaction.atomic():
                for template in template_id:
                    setAttr(_data, 'template_id', template)
                    operation_material, created = super().get_or_create(**_data)
                    if created:
                        _res = model_to_dict(operation_material)
                        setAttr(_res, 'material_askon__field0', operation_material.material_askon.field0 if operation_material.material_askon else None)
                        setAttr(_res, 'complex_name', operation_material.complex_name)
                        setAttr(_res, 'complex_gost', operation_material.complex_gost)
                        setAttr(_res, 'edizm__code', operation_material.edizm.code)
                        res.append(_res)
        else:
            operation_material, created = super().get_or_create(**_data)
            if created:
                _res = model_to_dict(operation_material)
                setAttr(_res, 'material_askon__field0', operation_material.material_askon.field0 if operation_material.material_askon else None)
                setAttr(_res, 'complex_name', operation_material.complex_name)
                setAttr(_res, 'complex_gost', operation_material.complex_gost)
                setAttr(_res, 'edizm__code', operation_material.edizm.code)
            res.append(_res)
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'template_id': record.template.id,
            'material_askon_id': record.material_askon.id if record.material_askon else None,
            'material_askon__field0': record.material_askon.field0 if record.material_askon else '',
            'material_id': record.material.id if record.material else None,
            'material__name': record.material.name if record.material else '',
            'complex_name': record.complex_name,
            'complex_gost': record.complex_gost,
            'edizm_id': record.edizm.id,
            'edizm__code': record.edizm.code,
            'edizm__name': record.edizm.name,
            'qty': record.qty,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operations_template_materialQuerySet(self.model, using=self._db)


class Operations_template_material(Hierarcy):
    template = ForeignKeyCascade(Operations_template_detail)
    material = ForeignKeyProtect(Materials, null=True, blank=True)
    material_askon = ForeignKeyProtect(Material_askon, null=True, blank=True)
    edizm = ForeignKeyProtect(Ed_izm, default=None)
    qty = DecimalField(max_digits=10, decimal_places=4, default=0.0)

    @property
    def complex_name(self):
        if self.material:
            return f'{self.material.materials_type.full_name}{self.material.full_name}'
        else:
            return ''

    @property
    def complex_gost(self):
        if self.material:
            if self.material.materials_type.gost:
                return f'{self.material.materials_type.gost} / {self.material.gost}'
            else:
                return self.material.gost if self.material else ''
        else:
            return ''

    objects = Operations_template_materialManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'
        constraints = [
            UniqueConstraint(fields=['template'], condition=Q(material=None) & Q(material_askon=None), name='Operations_template_material_unique_constraint_0'),
            UniqueConstraint(fields=['material', 'template'], condition=Q(material_askon=None), name='Operations_template_material_unique_constraint_1'),
            UniqueConstraint(fields=['material_askon', 'template'], condition=Q(material=None), name='Operations_template_material_unique_constraint_2'),
            UniqueConstraint(fields=['material', 'material_askon', 'template'], name='Operations_template_material_unique_constraint_3'),
        ]



# Generated by Django 2.2 on 2019-04-20 12:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0034_auto_20190419_0536'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_line',
            unique_together={('parent', 'child')},
        ),
        migrations.RemoveField(
            model_name='item_line',
            name='document',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-26 12:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0210_auto_20200226_1030'),
    ]

    # operations = [
    #     migrations.RemoveConstraint(
    #         model_name='resource',
    #         name='Resource_unique_constraint_0',
    #     ),
    #     migrations.RemoveConstraint(
    #         model_name='resource',
    #         name='Resource_unique_constraint_1',
    #     ),
    # ]



# Generated by Django 2.2.5 on 2019-09-18 01:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0063_auto_20190828_2056'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations_plan_refs',
            name='child',
        ),
        migrations.RemoveField(
            model_name='operations_plan_refs',
            name='parent',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='item',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='location',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='status',
        ),
        migrations.RemoveField(
            model_name='operationsplan',
            name='type',
        ),
        migrations.DeleteModel(
            name='Operations_plan_view',
        ),
        migrations.DeleteModel(
            name='Operation_plan_types',
        ),
        migrations.DeleteModel(
            name='Operations_plan_refs',
        ),
        migrations.DeleteModel(
            name='OperationsPlan',
        ),
    ]



import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction
from isc_common.bit import TurnBitOn
from isc_common.common import blinkString
from isc_common.common.mat_views import refresh_mat_view
from isc_common.datetime import DateTimeToStr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import DelProps
from isc_common.progress import managed_progress, ProgressDroped

from kaf_pas.kd.models.pathes import Pathes
from kaf_pas.production.models import progress_deleted

logger = logging.getLogger(__name__)


class UploadesQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class UploadesManager(AuditManager):

    @staticmethod
    def props():
        return BitField(flags=(
            ('confirmed', 'Подтверждено'),
        ), default=0, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'lastmodified': record.lastmodified,
            'path_id': record.path.id,
            'absolute_path': record.path.absolute_path,
            'editing': record.editing,
            'deliting': record.deliting,
            'props': record.props,
            'confirmed': record.props.confirmed,
        }
        return DelProps(res)

    def get_queryset(self):
        return UploadesQuerySet(self.model, using=self._db)

    @staticmethod
    def del_upload(id, user, doc_cnt=0, lotsman_cnt=0, document_id=None, last_step=False):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.ckk.models.item_document import Item_document
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.documents import Documents
        from kaf_pas.kd.models.documents_history import Documents_history
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from kaf_pas.kd.models.lotsman_document_attr_cross import Lotsman_document_attr_cross
        from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
        from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files
        from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs
        from kaf_pas.kd.models.uploades_documents import Uploades_documents
        from kaf_pas.kd.models.uploades_log import Uploades_log

        if settings.LOCKS.locked(f'UploadesManager.del_upload_{id}'):
            return

        settings.LOCKS.acquire(f'UploadesManager.del_upload_{id}')

        if document_id == None:
            for upload_document in Uploades_documents.objects.filter(upload_id=id):
                lotsman_cnt += Lotsman_documents_hierarcy.objects.filter(document=upload_document.document).count()
        else:
            lotsman_cnt = Lotsman_documents_hierarcy.objects.filter(document_id=document_id).count()

        upload = Uploades.objects.get(id=id)

        if document_id != None:
            upload_document = Uploades_documents.objects.get(id=document_id)

        qty = Uploades_documents.objects.filter(upload_id=id).count() + lotsman_cnt if document_id == None else 1 + lotsman_cnt
        with managed_progress(
                qty=qty,
                id=f'Remove_upload_{id}',
                user=user,
                message=f'Удаление закачки: {upload.path.drived_absolute_path} от {DateTimeToStr(upload.lastmodified)}' if document_id == None else f'Удаление документа: {upload_document.document.file_document}',
                title='Выполнено',
                props=TurnBitOn(0, 0)
        ) as progress:
            try:
                res = 0
                with transaction.atomic():
                    continue_cnt = 0

                    if document_id == None:
                        upload_query = Uploades_documents.objects.select_for_update().filter(upload_id=id)
                    else:
                        upload_query = Uploades_documents.objects.select_for_update().filter(id=document_id)

                    for upload_document in upload_query:

                        _continue = False
                        for item in Item.objects.filter(document=upload_document.document):
                            if ItemManager.delete_recursive(item_id=item.id, delete_lines=True, user=user, props=0, document=upload_document.document, show_progress=True) == 2:
                                _continue = True
                                continue_cnt += 1
                                break
                            else:
                                item.delete()

                        if _continue:
                            continue

                        deleted = Document_attr_cross.objects.filter(document=upload_document.document).delete()
                        # logger.debug(f'deleted: {deleted}')

                        for documents_history in Documents_history.objects.filter(new_document=upload_document.document):
                            documents_history.old_document.props |= Documents.props.relevant
                            documents_history.old_document.save()
                            deleted = documents_history.delete()
                            # logger.debug(f'deleted: {deleted}')

                        for thumb in Documents_thumb.objects.filter(document=upload_document.document):
                            Item_image_refs.objects.filter(thumb=thumb).delete()
                            deleted = thumb.delete()
                            # logger.debug(f'deleted: {deleted}')

                        for thumb10 in Documents_thumb10.objects.filter(document=upload_document.document):
                            Item_image_refs.objects.filter(thumb10=thumb10).delete()
                            deleted = thumb10.delete()
                            # logger.debug(f'deleted: {deleted}')

                        for lotsman_document in Lotsman_documents_hierarcy.objects.filter(document=upload_document.document):

                            for item in Item.objects.filter(lotsman_document=lotsman_document):
                                if ItemManager.delete_recursive(item_id=item.id, delete_lines=True, user=user, props=0, document=lotsman_document, show_progress=True) == 2:
                                    _continue = True
                                    continue_cnt += 1
                                    break
                                else:
                                    Item_document.objects.filter(item=item).delete()
                                    item.delete()

                            if _continue:
                                break

                            deleted = Lotsman_document_attr_cross.objects.filter(document=lotsman_document).delete()
                            # logger.debug(f'deleted: {deleted}')
                            deleted = Lotsman_document_attr_cross.objects.filter(parent_document=lotsman_document).delete()
                            # logger.debug(f'deleted: {deleted}')
                            deleted = Lotsman_documents_hierarcy_refs.objects.filter(child=lotsman_document).delete()
                            # logger.debug(f'deleted: {deleted}')
                            deleted = Lotsman_documents_hierarcy_refs.objects.filter(parent=lotsman_document).delete()
                            # logger.debug(f'deleted: {deleted}')

                            for thumb in Documents_thumb.objects.filter(lotsman_document=lotsman_document):
                                Item_image_refs.objects.filter(thumb=thumb).delete()
                                deleted = thumb.delete()
                                # logger.debug(f'deleted: {deleted}')

                            for thumb10 in Documents_thumb10.objects.filter(lotsman_document=lotsman_document):
                                Item_image_refs.objects.filter(thumb10=thumb10).delete()
                                deleted = thumb10.delete()
                                # logger.debug(f'deleted: {deleted}')

                            Lotsman_documents_hierarcy_files.objects.filter(lotsman_document=lotsman_document).delete()
                            deleted = lotsman_document.delete()
                            # logger.debug(f'deleted: {deleted}')
                            lotsman_cnt += 1
                            if progress.step() != 0:
                                raise ProgressDroped(progress_deleted)

                        if _continue:
                            continue

                        # logger.debug(f'deleted: {deleted}')
                        Uploades_documents.objects.filter(document=upload_document.document).delete()
                        Item_document.objects.filter(document=upload_document.document).delete()
                        Documents.objects.filter(id=upload_document.document.id).delete()

                        doc_cnt += 1
                        # logger.debug('Done.')
                        if progress.step() != 0:
                            raise ProgressDroped(progress_deleted)

                    if continue_cnt == 0:
                        deleted = Uploades_log.objects.filter(upload_id=id).delete()
                        # logger.debug(f'deleted: {deleted}')
                        if document_id == None:
                            res += Uploades.objects.filter(id=id).delete()[0]

                if doc_cnt > 0 and last_step:
                    progress.setContentsLabel(blinkString(text='Обновление представления "kd_documents_mview"', color='blue'))
                    refresh_mat_view('kd_documents_mview')

                if lotsman_cnt > 0 and last_step:
                    progress.setContentsLabel(blinkString(text='Обновление представления "kd_lotsman_documents_hierarcy_mview"', color='blue'))
                    refresh_mat_view('kd_lotsman_documents_hierarcy_mview')

                settings.LOCKS.release(f'UploadesManager.del_upload_{id}')
                return res, doc_cnt, lotsman_cnt
            except ProgressDroped as ex:
                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                settings.LOCKS.release(f'UploadesManager.del_upload_{id}')
                raise ex

            except Exception as ex:
                settings.LOCKS.release(f'UploadesManager.del_upload_{id}')
                raise ex

    def deleteFromRequest(self, request):
        from isc_common.auth.models.user import User

        ids = request.GET.getlist('ids')
        user = User.objects.get(username=request.GET.get('ws_channel').split('_')[1])

        res = 0
        doc_cnt = 0
        lotsman_cnt = 0

        for i in range(0, len(ids), 2):
            id = ids[i]
            visibleMode = ids[i + 1]

            if visibleMode != "none":
                res += super().filter(id=id).soft_delete(visibleMode=visibleMode)
            else:
                res, _doc_cnt, _lotsman_cnt = UploadesManager.del_upload(
                    id=id,
                    user=user,
                    doc_cnt=doc_cnt,
                    last_step=len(ids) / 2 == i,
                    lotsman_cnt=lotsman_cnt
                )
                doc_cnt += _doc_cnt
                lotsman_cnt += _lotsman_cnt
        return res

    def confirmationFromRequest(self, request):
        from isc_common.http.DSRequest import DSRequest

        request = DSRequest(request=request)
        data = request.get_data()
        user = request.user

        res = self.change_confirm(data=data, user=user)

        return dict(res=res)

    def change_confirm(self, data, user, kind='confirm'):
        from kaf_pas.kd.models.uploades_documents import Uploades_documents
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

        res = 0

        if isinstance(data.get('ids'), list):
            with managed_progress(
                    id=f'confirm_{user.id}' if kind == 'confirm' else f'unconfirm_{user.id}',
                    qty=0,
                    user=user,
                    message='Подсчет товарных позиций',
                    props=TurnBitOn(0, 0)
            ) as progress:
                try:
                    with transaction.atomic():
                        for upload in Uploades.objects.select_for_update().filter(id__in=data.get('ids')):
                            for uploades_document in Uploades_documents.objects.filter(upload=upload):
                                res += Item.objects.filter(document=uploades_document.document).count()

                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy.objects.filter(document=uploades_document.document):
                                    res += Item.objects.filter(lotsman_document=lotsman_documents_hierarcy).count()

                            if res == 0:
                                raise Exception(f'Товарных позиций не найдено.')

                            progress.setQty(qty=res)
                            if kind == 'confirm':
                                progress.setContentsLabel(content=blinkString(f'Подтверждение закачки {upload.path.drived_absolute_path} от {DateTimeToStr(upload.lastmodified, hours=3)}', blink=False, bold=True))
                            else:
                                progress.setContentsLabel(content=blinkString(f'Отмена Подтверждения закачки {upload.path.drived_absolute_path} от {DateTimeToStr(upload.lastmodified, hours=3)}', blink=False, bold=True))
                            for uploades_document in Uploades_documents.objects.filter(upload=upload):
                                for item in Item.objects.filter(document=uploades_document.document):
                                    if kind == 'confirm':
                                        item.props |= Item.props.confirmed
                                    else:
                                        item.props &= ~Item.props.confirmed
                                    item.save()
                                    if progress.step() != 0:
                                        raise ProgressDroped

                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy.objects.filter(document=uploades_document.document):
                                    for item in Item.objects.filter(lotsman_document=lotsman_documents_hierarcy):
                                        if kind == 'confirm':
                                            item.props |= Item.props.confirmed
                                        else:
                                            item.props &= ~Item.props.confirmed
                                        item.save()
                                        if progress.step() != 0:
                                            raise ProgressDroped

                            if kind == 'confirm':
                                upload.props |= Uploades.props.confirmed
                            else:
                                upload.props &= ~Uploades.props.confirmed
                            upload.save()
                        progress.sendMessage(type='refresh_uploads_grid')
                except ProgressDroped as ex:
                    from isc_common.models.deleted_progresses import Deleted_progresses
                    Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                    raise ex

        return res

    def unConfirmationFromRequest(self, request):
        from isc_common.http.DSRequest import DSRequest

        request = DSRequest(request=request)
        data = request.get_data()
        user = request.user

        res = self.change_confirm(data=data, user=user, kind='unconfirm')

        return dict(res=res)


class Uploades(AuditModel):
    path = ForeignKeyProtect(Pathes)

    props = UploadesManager.props()

    objects = UploadesManager()

    def __str__(self):
        return f"{self.id}, path: [{self.path}]"

    class Meta:
        verbose_name = 'Загрузки внешних данных'



# Generated by Django 3.0.3 on 2020-02-18 12:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0096_operation_operation'),
    ]

    # operations = [
    #     migrations.AlterModelOptions(
    #         name='operations_view',
    #         options={},
    #     ),
    # ]



# Generated by Django 3.0.3 on 2020-02-17 12:21

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0210_auto_20200212_0752'),
        ('production', '0191_auto_20200217_1221'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs',
            name='item_full_name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='launch_item_refs',
            name='item_refs',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item_refs'),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_MARK

urlpatterns = [

    path('Document_attributes_SPC_CLM_MARK/Fetch/', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Fetch),
    path('Document_attributes_SPC_CLM_MARK/Add', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Add),
    path('Document_attributes_SPC_CLM_MARK/Update', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Update),
    path('Document_attributes_SPC_CLM_MARK/Remove', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Remove),
    path('Document_attributes_SPC_CLM_MARK/Lookup/', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Lookup),
    path('Document_attributes_SPC_CLM_MARK/Info/', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Info),
    path('Document_attributes_SPC_CLM_MARK/Copy', document_attributes_SPC_CLM_MARK.Document_attributes_SPC_CLM_MARK_Copy),

]



from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.accounting.models.tmp_buffer import Tmp_buffer, Tmp_bufferManager


@JsonResponseWithException()
def Tmp_buffer_off_Fetch(request):
    from kaf_pas.accounting.models.buffers import Buffers

    req = DSRequest(request=request)
    Tmp_buffer.objects.filter(user=req.user).delete()
    for buffers in Buffers.objects.all():
        Tmp_buffer.objects.create(
            item=buffers.item,
            location=buffers.location,
            color=buffers.color,
            edizm=buffers.edizm,
            value=buffers.value,
            user=req.user,
        )

    return JsonResponse(
        DSResponse(
            request=request,
            data=Tmp_buffer.objects.
                filter(user=req.user).
                filter().
                get_range_rows1(
                request=request,
                function=Tmp_bufferManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_off_Add(request):
    return JsonResponse(DSResponseAdd(data=Tmp_buffer.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_off_Update(request):
    return JsonResponse(DSResponseUpdate(data=Tmp_buffer.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_off_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_off_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_off_Info(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Tmp_buffer_off_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Tmp_buffer.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.8 on 2019-12-13 12:52

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0088_auto_20191213_1252'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_documents',
            name='attr_type',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Тип атрибута'),
        ),
    ]



import logging

from django.conf import settings
from django.db import transaction, connection
from django.db.models import PositiveIntegerField, OneToOneField, CASCADE, UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.production.models.operations_item import Operations_item
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Operation_resourcesQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        if kwargs.get('resource'):
            setAttr(kwargs, 'resource_id', kwargs.get('resource').id)
            delAttr(kwargs, 'resource')

        if kwargs.get('location'):
            setAttr(kwargs, 'location_id', kwargs.get('location').id)
            delAttr(kwargs, 'location')

        if not kwargs.get('resource_id') and not kwargs.get('location_id'):
            raise Exception('Необходим хотябы один выбранный параметр.')
        delAttr(kwargs, 'resource__name')
        delAttr(kwargs, 'location__full_name')

        resource_id = kwargs.get('resource_id')
        if resource_id:
            setAttr(kwargs, 'location_id', Resource.objects.get(id=resource_id).location.id)

        setAttr(kwargs, '_operation', 'create')
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operation_resourcesManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def refresh_resource(apps=None, schema_editor=None):
        settings.LOCKS.acquire('Operation_resourcesManager.refresh_resource')
        sql_text = '''select  operationitem_id, count(*)
                    from production_operation_resources
                    group by operationitem_id
                    having count(*) > 1'''

        sql_items = f'''select id
            from production_operation_resources
            where operationitem_id = %s'''

        with transaction.atomic():
            with connection.cursor() as cursor:
                cursor.execute(sql_text)
                rows = cursor.fetchall()
                for row in rows:
                    operationitem_id, count = row
                    # print(f'launch_operationitem_id: {launch_operationitem_id}')
                    cursor.execute(sql_items, [operationitem_id])
                    rows_item = cursor.fetchall()

                    first_step = True
                    for item in rows_item:
                        id, = item
                        if not first_step:
                            # print(f'id: {id}')
                            res = Operation_resources.objects.filter(id=id).delete()
                            # print(res)
                        else:
                            first_step = False
        settings.LOCKS.release('Operation_resourcesManager.refresh_resource')

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        operationitem_id = _data.get('operationitem_id')

        res = []
        if isinstance(operationitem_id, list):
            delAttr(_data, 'operationitem_id')
            with transaction.atomic():
                for operationitem in operationitem_id:
                    setAttr(_data, 'operationitem_id', operationitem)
                    delAttr(_data, 'location__full_name')
                    delAttr(_data, 'resource__name')
                    operation_resource, created = super().get_or_create(**_data)
                    if created:
                        _res = model_to_dict(operation_resource)
                        setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                        setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
                        res.append(_res)
        else:
            delAttr(_data, 'location__full_name')
            delAttr(_data, 'resource__name')
            operation_resource, created = super().get_or_create(**_data)
            if created:
                _res = model_to_dict(operation_resource)
                setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
                res.append(_res)
        return res

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        self._remove_prop(data, removed)
        _data = self.clone_data(data)
        delAttr(_data, 'id')
        res = super().filter(id=data.get('id')).update(**_data)
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operationitem_id': record.operationitem.id if record.operationitem else None,
            'resource_id': record.resource.id if record.resource else None,
            'resource__code': record.resource.code if record.resource else None,
            'resource__name': record.resource.name if record.resource else None,
            'location_id': record.location.id if record.location else None,
            'location__code': record.location.code if record.location else None,
            'location__name': record.location.name if record.location else None,
            'location__full_name': record.location.full_name if record.location else None,
            'complex_name': record.complex_name,
            'batch_size': record.batch_size,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_resourcesQuerySet(self.model, using=self._db)


class Operation_resources(AuditModel):
    operationitem = OneToOneField(Operations_item, on_delete=CASCADE)
    resource = ForeignKeyProtect(Resource, null=True, blank=True)
    location = ForeignKeyProtect(Locations, null=True, blank=True)
    batch_size = PositiveIntegerField(default=1)

    @property
    def complex_name(self):
        return f'{self.resource.location.full_name}{self.resource.name}' if self.resource else None

    objects = Operation_resourcesManager()

    def __str__(self):
        return f"{self.id}, operationitem: [{self.operationitem}], resource: [{self.resource}], location: [{self.location}]"

    def __repr__(self):
        return str(self.resource) if self.resource else str(self.location)

    class Meta:
        verbose_name = 'Кросс таблица'
        constraints = [
            UniqueConstraint(fields=['operationitem'], condition=Q(location=None) & Q(resource=None), name='Operation_resources_unique_constraint_0'),
            UniqueConstraint(fields=['operationitem', 'resource'], condition=Q(location=None), name='Operation_resources_unique_constraint_1'),
            UniqueConstraint(fields=['location', 'operationitem'], condition=Q(resource=None), name='Operation_resources_unique_constraint_2'),
            UniqueConstraint(fields=['location', 'operationitem', 'resource'], name='Operation_resources_unique_constraint_3'),
        ]




from django.urls import path

from kaf_pas.production.views import operations

urlpatterns = [

    path('Operations/Fetch/', operations.Operations_Fetch),
    path('Operations/Add', operations.Operations_Add),
    path('Operations/Update', operations.Operations_Update),
    path('Operations/Remove', operations.Operations_Remove),
    path('Operations/Lookup/', operations.Operations_Lookup),
    path('Operations/Info/', operations.Operations_Info),

]



# Generated by Django 2.2.7 on 2019-11-17 11:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0089_auto_20191117_1110'),
        ('sales', '0112_auto_20191117_0448'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Status_launch',
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import production_order

urlpatterns = [

    path('Production_order/Fetch/', production_order.Production_order_Fetch),
    path('Production_order/FetchDetail/', production_order.Production_order_FetchDetail),
    path('Production_order/FetchLocations/', production_order.Production_order_FetchLocations),
    path('Production_order/FetchLevels/', production_order.Production_order_FetchLevels),
    path('Production_order/FetchExecutorsLocation/', production_order.Production_order_FetchExecutorsLocation),
    path('Production_order/Add', production_order.Production_order_Add),
    path('Production_order/Update', production_order.Production_order_Update),
    path('Production_order/Remove', production_order.Production_order_Remove),
    path('Production_order/Lookup/', production_order.Production_order_Lookup),
    path('Production_order/Info/', production_order.Production_order_Info),
    path('Production_order/Copy', production_order.Production_order_Copy),
    path('Production_order/CheckStatus', production_order.Production_order_CheckStatus),
    path('Production_order/SetPrevStatus', production_order.Production_order_SetPrevStatus),
    path('Production_order/SetStartStatus', production_order.Production_order_SetStartStatus),
    path('Production_order/getValue_made/', production_order.Production_order_getValue_made),

]



# Generated by Django 2.2.8 on 2019-12-07 10:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0163_locations_users'),
        ('production', '0157_auto_20191207_1031'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('item', 'num')},
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager
from kaf_pas.kd.models.document_attributes_view import Document_attributes_view


@JsonResponseWithException()
def Document_attributes_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Document_attributes_Fetch_STMP_2(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='STMP_2').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_FORMAT(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_FORMAT').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_ZONE(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_ZONE').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_POS(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_POS').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_MARK(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_MARK').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_NAME(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_NAME').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_COUNT(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_COUNT').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_NOTE(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_NOTE').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_MASSA(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_MASSA').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_MATERIAL(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_MATERIAL').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_USER(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_USER').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_KOD(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_KOD').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Fetch_SPC_CLM_FACTORY(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Add(request):
    return JsonResponse(DSResponseAdd(data=Document_attributes.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Update(request):
    return JsonResponse(DSResponseUpdate(data=Document_attributes.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_Info(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.8 on 2019-12-12 18:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0167_auto_20191212_1745'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Item_refs_hierarcy',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0080_remove_pathes_props'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document_attributes',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='documents',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='documents_history',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='documents_thumb',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='documents_thumb10',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='pathes',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='uploades',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='uploades_documents',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='uploades_log',
            name='id_old',
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-29 18:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0080_auto_20191028_1406'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ready_2_launch',
            name='notes',
        ),
    ]



import logging
import os

from django.db import transaction

from isc_common import setAttr
from isc_common.common.functions import delete_dbl_spaces
from kaf_pas.kd.models.document_attributes_view import Document_attributes_view

logger = logging.getLogger(__name__)

class DocumentManagerExt:

    def __init__(self, logger):
        self.logger = logger

    def rec_image_cwd(self, item_id, STMP_2):
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.ckk.models.item import Item
        for item_image in Item_image_refs.objects.select_related('item').filter(
                item__props__in=[~Item.props.relevant & Item.props.from_cdw, ~Item.props.relevant & Item.props.from_pdf],
                item__STMP_2__value_str__delete_dbl_spaces=STMP_2):
            _, created = Item_image_refs.objects.get_or_create(
                item_id=item_id,
                thumb_id=item_image.thumb_id,
                thumb10_id=item_image.thumb10_id
            )

    def link_image_to_item(self, item):
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10

        document = item.document
        if document != None:
            for document_thumb in Documents_thumb.objects.filter(document=document):
                Item_image_refs.objects.get_or_create(item=item, thumb=document_thumb)

            for document_thumb10 in Documents_thumb10.objects.filter(document=document):
                Item_image_refs.objects.get_or_create(item=item, thumb10=document_thumb10)

    def link_image_to_lotsman_item(self, lotsman_document, item):
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10

        if lotsman_document != None:
            for document_thumb in Documents_thumb.objects.filter(lotsman_document_id=lotsman_document.id):
                Item_image_refs.objects.get_or_create(item=item, thumb=document_thumb)

            for document_thumb10 in Documents_thumb10.objects.filter(lotsman_document_id=lotsman_document.id):
                Item_image_refs.objects.get_or_create(item=item, thumb10=document_thumb10)

    def make_spw(self, document):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.document_attributes import Document_attributes
        from kaf_pas.kd.models.documents import Documents
        from kaf_pas.kd.models.spw_attrs import Spw_attrsQuerySet
        from kaf_pas.system.models.contants import Contants
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.ckk.models.item_document import Item_document

        top_level = Contants.objects.get(code='audo_top_level')

        with transaction.atomic():
            if document.file_document.lower().find('мусор') == -1:

                try:
                    STMP_1 = Document_attributes_view.objects.get(document=document, attr_type__code='STMP_1')
                except Document_attributes_view.DoesNotExist:
                    STMP_1 = None

                try:
                    STMP_2 = Document_attributes_view.objects.get(document=document, attr_type__code='STMP_2')
                except Document_attributes_view.DoesNotExist:
                    STMP_2 = None

                if STMP_1 != None or STMP_2 != None:
                    props = Item.props.relevant | Item.props.from_spw

                    if STMP_1 == None:
                        item_query = Item.objects.filter(
                            STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2.value_str),
                            props=props
                        )
                    elif STMP_2 == None:
                        item_query = Item.objects.filter(
                            STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1.value_str),
                            props=props
                        )
                    else:
                        item_query = Item.objects.filter(
                            STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1.value_str),
                            STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2.value_str),
                            props=props
                        )

                    parent_count = 0
                    parent = None
                    for item in item_query:
                        parent_count = Item_refs.objects.filter(parent=item).count()
                        parent = item
                        if parent_count > 0:
                            break

                    if parent == None:
                        parent = Item.objects.create(
                            STMP_1_id=STMP_1.id if STMP_1 else None,
                            STMP_2_id=STMP_2.id if STMP_2 else None,
                            props=props,
                            document=document,
                            version=ItemManager.get_verstion(STMP_1=STMP_1, STMP_2=STMP_2, props=Item.props.from_spw)
                        )

                        Item_refs.objects.create(parent_id=int(top_level.value), child=parent)
                        self.link_image_to_item(parent)
                    else:
                        if parent_count == 0:
                            Item_document.objects.get_or_create(item=parent, document=document)

                    if STMP_2 != None:
                        self.rec_image_cwd(parent.id, delete_dbl_spaces(STMP_2.value_str))

                    query_spw = Document_attributes_view.objects.filter(document=document).order_by(*['position_in_document'])
                    specification = Spw_attrsQuerySet.make_specification(queryResult=query_spw)

                    SPC_CLM_MARK_old = None
                    for line_specification in specification:
                        SPC_CLM_NAME = delete_dbl_spaces(line_specification.get('SPC_CLM_NAME'))
                        SPC_CLM_NAME_ID = line_specification.get('SPC_CLM_NAME_ID')

                        SPC_CLM_MARK = delete_dbl_spaces(line_specification.get('SPC_CLM_MARK'))
                        SPC_CLM_MARK_ID = line_specification.get('SPC_CLM_MARK_ID')

                        if SPC_CLM_NAME_ID != None or SPC_CLM_MARK_ID != None:
                            if isinstance(SPC_CLM_MARK, str) and SPC_CLM_MARK.startswith('-') and isinstance(SPC_CLM_MARK_ID, int):
                                _SPC_CLM_MARK = f'{SPC_CLM_MARK_old}{SPC_CLM_MARK}'
                                for document_attribute in Document_attributes.objects.filter(id=SPC_CLM_MARK_ID):
                                    try:
                                        _document_attribute = Document_attributes.objects.get(
                                            attr_type=document_attribute.attr_type,
                                            value_str=_SPC_CLM_MARK
                                        )

                                        try:
                                            document_attr_cross = Document_attr_cross.objects.filter(
                                                attribute=document_attribute,
                                                document=document)[0]
                                            setAttr(line_specification, 'SPC_CLM_MARK_ID', _document_attribute.id)
                                        except IndexError:
                                            Document_attr_cross.objects.create(
                                                attribute=_document_attribute,
                                                document=document
                                            )
                                        setAttr(line_specification, 'SPC_CLM_MARK_ID', _document_attribute.id)
                                    except Document_attributes.DoesNotExist:
                                        document_attribute.value_str = _SPC_CLM_MARK
                                        document_attribute.save()

                            elif isinstance(SPC_CLM_MARK, str):
                                SPC_CLM_MARK_old = SPC_CLM_MARK

                            try:
                                if SPC_CLM_MARK == None:
                                    child = Item.objects.filter(
                                        STMP_1__value_str__delete_dbl_spaces=SPC_CLM_NAME,
                                        props=Item.props.relevant
                                    )[0]
                                elif SPC_CLM_NAME == None:
                                    child = Item.objects.filter(
                                        STMP_2__value_str__delete_dbl_spaces=SPC_CLM_MARK,
                                        props=Item.props.relevant
                                    )[0]
                                else:
                                    child = Item.objects.filter(
                                        STMP_1__value_str__delete_dbl_spaces=SPC_CLM_NAME,
                                        STMP_2__value_str__delete_dbl_spaces=SPC_CLM_MARK,
                                        props=Item.props.relevant
                                    )[0]
                                Item_document.objects.get_or_create(item=child, document=document)

                            except IndexError:
                                d = dict(
                                    STMP_1_id=SPC_CLM_NAME_ID,
                                    STMP_2_id=SPC_CLM_MARK_ID,
                                    props=Item.props.relevant | Item.props.from_spw | Item.props.for_line,
                                )
                                logger.debug(f'd: {d}')
                                child, created = Item.objects.get_or_create(**d, defaults=dict(document=document))

                                if not created:
                                    Item_document.objects.get_or_create(item=child, document=document)
                                    logger.debug(f'Not Created: {child}')
                                else:
                                    logger.debug(f'Created: {child}')

                            if parent.id != child.id:
                                Item_refs.objects.get_or_create(parent=parent, child=child)

                                defaults = dict(
                                    parent=parent,
                                    child=child,
                                    SPC_CLM_FORMAT_id=line_specification.get('SPC_CLM_FORMAT_ID'),
                                    SPC_CLM_ZONE_id=line_specification.get('SPC_CLM_ZONE_ID'),
                                    SPC_CLM_POS_id=line_specification.get('SPC_CLM_POS_ID'),
                                    SPC_CLM_MARK_id=line_specification.get('SPC_CLM_MARK_ID'),
                                    SPC_CLM_NAME_id=line_specification.get('SPC_CLM_NAME_ID'),
                                    SPC_CLM_COUNT_id=line_specification.get('SPC_CLM_COUNT_ID'),
                                    SPC_CLM_NOTE_id=line_specification.get('SPC_CLM_NOTE_ID'),
                                    SPC_CLM_MASSA_id=line_specification.get('SPC_CLM_MASSA_ID'),
                                    SPC_CLM_MATERIAL_id=line_specification.get('SPC_CLM_MATERIAL_ID'),
                                    SPC_CLM_USER_id=line_specification.get('SPC_CLM_USER_ID'),
                                    SPC_CLM_KOD_id=line_specification.get('SPC_CLM_KOD_ID'),
                                    SPC_CLM_FACTORY_id=line_specification.get('SPC_CLM_FACTORY_ID'),

                                    section=line_specification.get('section'),
                                    subsection=line_specification.get('subsection'),
                                )

                                item_line, _ = Item_line.objects.get_or_create(parent=parent, child=child, defaults=defaults)
                                # print(f'Create Item_line: {item_line}')

                else:
                    pass

            document.props |= Documents.props.beenItemed
            document.save()

    def make_cdw(self, document):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.kd.models.documents import Documents
        from kaf_pas.system.models.contants import Contants
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.ckk.models.item_document import Item_document

        top_level = Contants.objects.get(code='audo_top_level')

        with transaction.atomic():
            if document.file_document.lower().find('мусор') == -1:

                try:
                    STMP_1 = Document_attributes_view.objects.get(document=document, attr_type__code='STMP_1')
                except Document_attributes_view.DoesNotExist:
                    STMP_1 = None

                try:
                    STMP_2 = Document_attributes_view.objects.get(document=document, attr_type__code='STMP_2')
                except Document_attributes_view.DoesNotExist:
                    STMP_2 = None

                if STMP_1 != None or STMP_2 != None:
                    cnt = 0

                    for item in Item.objects.filter(
                            STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1.value_str) if STMP_1 else None,
                            STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2.value_str) if STMP_2 else None,
                            props=Item.props.relevant | Item.props.from_cdw,
                    ):
                        self.link_image_to_item(item)
                        Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                        Item_document.objects.get_or_create(item=item, document=document)
                        cnt += 1

                    if cnt == 0:
                        item = Item.objects.create(
                            STMP_1_id=STMP_1.id if STMP_1 else None,
                            STMP_2_id=STMP_2.id if STMP_2 else None,
                            props=Item.props.relevant | Item.props.from_cdw,
                            document=document,
                            version=ItemManager.get_verstion(STMP_1=STMP_1, STMP_2=STMP_2, props=Item.props.from_cdw)
                        )

                        Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                        self.link_image_to_item(item)

                else:
                    pass

                document.props |= Documents.props.beenItemed
                document.save()

    def make_pdf(self, document, STMP_1_type, STMP_2_type):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.ckk.models.item_document import Item_document
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.kd.models.document_attributes import Document_attributes
        from kaf_pas.kd.models.documents import Documents
        from kaf_pas.system.models.contants import Contants

        top_level = Contants.objects.get(code='audo_top_level')

        with transaction.atomic():
            if document.file_document.lower().find('мусор') == -1:
                full_path = document.file_document
                _, file_name = os.path.split(full_path)
                file_name_part = file_name.split(' - ')
                if len(file_name_part) == 2:

                    STMP_1, ext = os.path.splitext(file_name_part[1])
                    STMP_2 = file_name_part[0]
                else:
                    STMP_1 = file_name.strip().replace('.pdf', '').replace('.PDF', '')
                    STMP_2 = STMP_1

                STMP_1_attr, _ = Document_attributes.objects.get_or_create(value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1), attr_type=STMP_1_type)
                STMP_2_attr, _ = Document_attributes.objects.get_or_create(value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2), attr_type=STMP_2_type)

                cnt = 0

                for item in Item.objects.filter(
                        STMP_1__value_str__delete_dbl_spaces=STMP_1,
                        STMP_2__value_str__delete_dbl_spaces=STMP_2,
                        props=Item.props.relevant | Item.props.from_pdf,
                ):
                    self.link_image_to_item(item)
                    Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                    Item_document.objects.get_or_create(item=item, document=document)
                    cnt += 1

                if cnt == 0:
                    item = Item.objects.create(
                        STMP_1_id=STMP_1_attr,
                        STMP_2_id=STMP_2_attr,
                        props=Item.props.relevant | Item.props.from_pdf,
                        document=document,
                        version=ItemManager.get_verstion(STMP_1=STMP_1_attr, STMP_2=STMP_2_attr, props=Item.props.from_pdf)
                    )

                    Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                    self.link_image_to_item(item)

            document.props |= Documents.props.beenItemed
            document.save()



from django.urls import path

from kaf_pas.kd.views import lotsman_documents_hierarcy

urlpatterns = [

    path('Lotsman_documents_hierarcy/Fetch/', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Fetch),
    path('Lotsman_documents_hierarcy/Fetch1/', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Fetch1),
    path('Lotsman_documents_hierarcy_doc/Fetch/', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_doc_Fetch),
    path('Lotsman_documents_hierarcy/Add', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Add),
    path('Lotsman_documents_hierarcy/Update', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Update),
    path('Lotsman_documents_hierarcy/Remove', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Remove),
    path('Lotsman_documents_hierarcy/Lookup/', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Lookup),
    path('Lotsman_documents_hierarcy/Info/', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Info),
    path('Lotsman_documents_hierarcy/Copy', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_Copy),
    path('Lotsman_documents_hierarcy/MakeItem', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_MakeItem),
    path('Lotsman_documents_hierarcy/ReloadDoc', lotsman_documents_hierarcy.Lotsman_documents_hierarcy_ReloadDoc),
    path('Lotsman_documents_hierarcy/RefreshMView', lotsman_documents_hierarcy.Lotsman_documents_hierarcys_RefreshMView),
]
